/**
	$Id: SlotManager.java 5447 2010-10-19 11:02:17Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.jobagent.slotmanager;

import net.sf.basedb.clients.jobagent.Agent;
import net.sf.basedb.core.Job;

/**
	A slot manager is responsible for slot assigment to jobs. This is an important
	part of the job agent since a job may not be executed without a slot has been
	assigned to it and the slot manager ususally has a limited number of slots.
	The default slot manager for a job agent is the {@link InternalSlotManager}.
	It is also possible for several job agents to cooperate on slot assignment by
	using {@link MasterSlotManager} and {@link RemoteSlotManager}.
	
	@author Nicklas
	@since 2.16
	@base.modified $Date: 2010-10-19 13:02:17 +0200 (ti, 19 okt 2010) $
*/
public interface SlotManager
{
	/**
		Initialize the slot manager. This method is called once when the
		job agent is starting up. The slot manager may read configuration
		settings, and intialize local resources.
		
		@param agent The job agent
	*/
	public void init(Agent agent);
	
	/**
		This method is called when the job agent is shutting down. The
		slot manager should clean up any resources it is using.
	*/
	public void close();
	
	/**
		Get a slot for the specified job. If no free slot is available 
		null is returned.
		@param job The job that the job agent wants to execute
		@return A slot or null if no slot is available
	*/
	public Slot getSlot(Job job);
	
	/**
		Called by the job agent when the job has finished exeuting. 
		
		@param slot The slot that was assigned to the job
	*/
	public void releaseSlot(Slot slot);
	
	
}
