/*
	$Id: ScrollIterator.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import org.hibernate.ScrollableResults;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
	This is an iterator view of the result from a query
	using the Hibernate <code>ScrollableResults</code> object.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
*/
class ScrollIterator<E>
	implements Iterator<E>
{

	private final Class<E> dataClass;
	private final ScrollableResults results;
	private boolean isClosed;
	private E nextElement;

	ScrollIterator(Class<E> dataClass, ScrollableResults results)
	{
		assert dataClass != null : "dataClass == null";
		assert results != null : "results == null";
		this.dataClass = dataClass;
		this.results = results;
		this.isClosed = false;
		this.nextElement = null;
	}
	
	/*
		From the Iterator interface
		---------------------------
	*/
	@Override
	public boolean hasNext()
	{
		if (nextElement == null)
		{
			if (!isClosed()) 
			{
				if (results.next())
				{
					nextElement = dataClass.cast(results.get(0));
				}
				else
				{
					close();
				}
			}
		}
		return nextElement != null;
	}
	@Override
	public E next()
	{
		if (!hasNext()) throw new NoSuchElementException();
		E temp = nextElement;
		nextElement = null;
		return temp;
	}
	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	@Override
	public void remove()
	{
		throw new UnsupportedOperationException();
	}
	// ---------------------------------

	/**
		Close the results and return resources.
	*/
	public void close()
	{
		if (!isClosed()) results.close();
		isClosed = true;
	}
	/**
		Check if the results has been closed.
	*/
	public boolean isClosed()
	{
		return isClosed;
	}
	
}
