/*
	$Id: AndRestriction.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.util.EqualsHelper;

/**
	Combine one or more restrictions with AND: r[0] AND r[1] AND ...

	@author Nicklas
	@version 2.0
	@see Restrictions#and(Restriction[])
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
*/
public class AndRestriction
	implements Restriction
{
	private final Restriction[] restrictions;
	private int hashCode = 0;

	AndRestriction(Restriction... restrictions)
	{
		this.restrictions = restrictions;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		StringBuilder sb = new StringBuilder();
		sb.append("(");
		int i = 0;
		for(Restriction r : restrictions)
		{
			if (i > 0) sb.append(" AND ");
			sb.append(r.toQl(query, dc));
			i++;
		}
		sb.append(")");
		return sb.toString();
	}
	/**
		@return The restrictions
	*/
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.unmodifiableList(Arrays.asList(restrictions));
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		StringBuilder sb = new StringBuilder();
		sb.append("(");
		int i = 0;
		for(Restriction r : restrictions)
		{
			if (i > 0) sb.append(" AND ");
			sb.append(r.toString());
			i++;
		}
		sb.append(")");
		return sb.toString();
	}
	/**
		This restriction is equal to another AndRestriction if both
		contains the same restrictions (disregarding order).
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		AndRestriction o = (AndRestriction)other;
		return EqualsHelper.invariantEquals(restrictions, o.restrictions);
	}
	@Override
	public int hashCode()
	{
		if (hashCode == 0)
		{
			hashCode = EqualsHelper.hashCode(restrictions);
		}
		return hashCode;
	}
	// -------------------------------------------

}

