/**
	$Id: CelValidationFactory.java 5623 2011-05-06 12:09:13Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.affymetrix;

import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.util.extensions.ActionFactory;
import net.sf.basedb.util.extensions.InvokationContext;

/**
	Action factory for creating CEL file validators.
	
	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-05-06 14:09:13 +0200 (fr, 06 maj 2011) $
*/
public class CelValidationFactory
	implements ActionFactory<CelValidationAction>
{

	public CelValidationFactory()
	{}
	
	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@Override
	public boolean prepareContext(InvokationContext<? super CelValidationAction> context)
	{
		Object item = context.getClientContext().getCurrentItem();
		boolean isInContext = false;
		if (item instanceof RawBioAssay)
		{
			// Validation context
			RawBioAssay rba = (RawBioAssay)item;
			isInContext = AffymetrixUtil.isAffymetrix(rba);
		}
		else if (item instanceof DataFileType)
		{
			// Do we support validating the given file type?
			DataFileType fileType = (DataFileType)item;
			isInContext = DataFileType.AFFYMETRIX_CEL.equals(fileType.getExternalId());
		}
		return isInContext;
	}


	@Override
	public CelValidationAction[] getActions(InvokationContext<? super CelValidationAction> context)
	{
		Object item = context.getClientContext().getCurrentItem();
		RawBioAssay ownerItem = null;
		if (item instanceof RawBioAssay)
		{
			ownerItem = (RawBioAssay)item;
		}
		
		CelValidationAction[] actions = null;
		if (ownerItem != null)
		{
			DbControl dc = context.getClientContext().getDbControl();
			actions = new CelValidationAction[] { new CelValidationAction(dc, ownerItem) };
		}
		return actions;
	}
	// -----------------------------------

}
