/**
	$Id: NullNodeLoader.java 6046 2012-04-17 10:46:51Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.Node;

/**
	A node loader implementation that doesn't load any nodes.
	This may be useful for {@link NodeLoaderFactory} implementations
	that wish to handle missing node loaders in the {@link 
	NodeLoaderFactory#createNodeLoader(Object)} method differently. Normally this should
	result in an {@link ItemNotFoundException}, but an alternative approach may
	be to return this loader which will effectively make an end-point in the
	node tree. This node loader can't be used for root nodes and will throw
	an {@link UnsupportedOperationException} if {@link 
	#createRootNode(DbControl, OverviewContext, Object)} is called.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2012-04-17 12:46:51 +0200 (ti, 17 apr 2012) $
*/
public class NullNodeLoader<I>
	implements NodeLoader<I>
{

	public NullNodeLoader()
	{}

	/*
		From the NodeLoader interface
		-----------------------------
	*/
	/**
		@return Always "null"
	*/
	@Override
	public Node createForwardNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		return null;
	}

	/**
		@return Always "null"
	*/
	@Override
	public Node createPropertyNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		return null;
	}

	/**
		@return Always "null"
	*/
	@Override
	public Node createReverseNode(DbControl dc, OverviewContext context, Node childNode)
	{
		return null;
	}
	
	/**
		@throws UnsupportedOperationException Always
	*/
	@Override
	public Node createRootNode(DbControl dc, OverviewContext context, I item)
	{
		throw new UnsupportedOperationException();
	}
	
	/**
		@return Always "false"
	*/
	@Override
	public boolean loadChildNodes(DbControl dc, OverviewContext context, Node node)
	{
		node.setChildrenLoaded();
		return false;
	}
	// -------------------------------------

}
