/*
	$Id: TestAnnotation.java 6921 2015-05-26 13:58:06Z nicklas $
	
	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import net.sf.basedb.core.*;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;

public class TestAnnotation
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing annotations");
		write_header();
		
		int celsiusId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.TEMPERATURE), "C");
		int fahrenheitId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.TEMPERATURE), "F");
		int meterId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.LENGTH), "m");
		int millimeterId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.LENGTH), "mm");

		// Create annotation types and array design
		int intId = TestAnnotationType.test_create(null, Type.INT, 0, 10, null, Item.ARRAYDESIGN, 1, null, true);
		int longId = TestAnnotationType.test_create(null, Type.LONG, 0, 1056567756767l, null, Item.ARRAYDESIGN, 1, null, true);
		int floatId = TestAnnotationType.test_create("Temperature", Type.FLOAT, celsiusId, 10.1f, null, Item.ARRAYDESIGN, 1, null, true);
		int doubleId = TestAnnotationType.test_create("Size", Type.DOUBLE, meterId, 8.13454646d, null, Item.ARRAYDESIGN, 1, null, true);
		int dateId = TestAnnotationType.test_create(null, Type.DATE, 0, new Date(), null, Item.ARRAYDESIGN, 1, null, true);
		int timestampId = TestAnnotationType.test_create(null, Type.TIMESTAMP, 0, new Date(), null, Item.ARRAYDESIGN, 1, null, true);
		int textId = TestAnnotationType.test_create(null, Type.TEXT, 0, "default value", null, Item.ARRAYDESIGN, 1, null, true);
		int stringId = TestAnnotationType.test_create(null, Type.STRING, 0, "default value", null, Item.ARRAYDESIGN, 1, null, true);
		int booleanId = TestAnnotationType.test_create(null, Type.BOOLEAN, 0, true, null, Item.ARRAYDESIGN, 1, null, true);
		int enumId = TestAnnotationType.test_create(null, Type.INT, 0, 3, null, Item.ARRAYDESIGN, 1, new Object[] { 1, 2, 3, 4, 5 }, false);
		int arrayDesignId = TestArrayDesign.test_create(Platform.GENERIC, true);
		int arrayDesignId2 = TestArrayDesign.test_create(Platform.AFFYMETRIX, true);

		// Annotate the array design and list the annotations
		int intAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, intId, 0, 10);
		int longAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, longId, 0, 1023456789012345l);
		int floatAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, floatId, celsiusId, 11.365789456857f);
		int doubleAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, doubleId, meterId, 11.365789456857d);
		int floatAnnotationId2 = test_annotatate(Item.ARRAYDESIGN, arrayDesignId2, floatId, fahrenheitId, 11.365789456857f);
		int doubleAnnotationId2 = test_annotatate(Item.ARRAYDESIGN, arrayDesignId2, doubleId, millimeterId, 11.365789456857d);
		int dateAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, dateId, 0, new Date());
		int timestampAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, timestampId, 0, new Date());
		int textAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, textId, 0, "a text annotation");
		int stringAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, stringId, 0, "a string annotation");
		int booleanAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, booleanId, 0, false);
		int enumAnnotationId = test_annotatate(Item.ARRAYDESIGN, arrayDesignId, enumId, 0, 4);
		test_list_annotations(Item.ARRAYDESIGN, arrayDesignId, null, 10);
		test_list_annotations(Item.ARRAYDESIGN, arrayDesignId2, Annotation.Source.PRIMARY, 2);

		// Test: inherit and clone annotations
		int rawBioAssayId = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", null, 0, 0, 0, 0, arrayDesignId, false);
		test_inherit_annotation(Item.RAWBIOASSAY, rawBioAssayId, intAnnotationId, false);
		int clonedId = test_inherit_annotation(Item.RAWBIOASSAY, rawBioAssayId, stringAnnotationId, true);
		test_modify_clone(clonedId, 0, "a cloned annotation");
		test_list_annotations(Item.RAWBIOASSAY, rawBioAssayId, null, 2);
		
		test_inherit_annotationset(Item.RAWBIOASSAY, rawBioAssayId, Item.ARRAYDESIGN, arrayDesignId, 8);
		test_list_annotations(Item.RAWBIOASSAY, rawBioAssayId, Annotation.Source.INHERITED, 9);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Remove an annotation that is inherited
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, intId);
		test_list_annotations(Item.RAWBIOASSAY, rawBioAssayId, null, 9);
		
		TestRawBioAssay.test_delete(rawBioAssayId);

		// Remove the annotations
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, intId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, longId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, floatId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, doubleId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, dateId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, timestampId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, textId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, stringId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, booleanId);
		test_remove_annotation(Item.ARRAYDESIGN, arrayDesignId, enumId);
		test_list_annotations(Item.ARRAYDESIGN, arrayDesignId, null, 0);

		// Delete annotation types and array design
		TestArrayDesign.test_delete(arrayDesignId);
		TestArrayDesign.test_delete(arrayDesignId2);
		TestAnnotationType.test_delete(intId);
		TestAnnotationType.test_delete(longId);
		TestAnnotationType.test_delete(floatId);
		TestAnnotationType.test_delete(doubleId);
		TestAnnotationType.test_delete(dateId);
		TestAnnotationType.test_delete(timestampId);
		TestAnnotationType.test_delete(textId);
		TestAnnotationType.test_delete(stringId);
		TestAnnotationType.test_delete(booleanId);
		TestAnnotationType.test_delete(enumId);

		write("++Testing annotations "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static int test_annotatate(Item itemType, int itemId, int annotationTypeId, int unitId, Object... value)
	{
		if (itemId == 0 || annotationTypeId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable annotatable = (Annotatable)itemType.getById(dc, itemId);
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);

			AnnotationSet as = annotatable.getAnnotationSet();
			Annotation a = as.getAnnotation(at);
			
			List<Object> l = new ArrayList<Object>();
			l.addAll(Arrays.asList(value));
			Unit unit = unitId != 0 ? Unit.getById(dc, unitId) : null;
			a.setValues(l, unit);
			dc.commit();
			id = a.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(a, false);
			write_item(0, a);
			write("--Create annotation OK");
		}
		catch (Throwable ex)
		{
			write("--Create annotation FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_remove_annotation(Item itemType, int itemId, int annotationTypeId)
	{
		if (itemId == 0 || annotationTypeId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable annotatable = (Annotatable)itemType.getById(dc, itemId);
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
			AnnotationSet as = annotatable.getAnnotationSet();
			as.removeAnnotation(at);
			dc.commit();
			write("--Remove annotation OK");
		}
		catch (Throwable ex)
		{
			write("--Remove annotation FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list_annotations(Item itemType, int itemId, Annotation.Source source, int expectedResults)
	{
		DbControl dc = null;
		Annotatable at = null;
		try
		{
			dc = TestUtil.getDbControl();
			at = (Annotatable)itemType.getById(dc, itemId);
			List<Annotation> l = Collections.emptyList();
			if (at.isAnnotated())
			{
				l = at.getAnnotationSet().getAnnotations(source).list(dc);
			}
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List " + (source==null ? "ALL":source)+" annotations for " + at + " OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List " + (source==null ? "ALL":source)+" annotations for " + at + " FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tAnnotation type\tAnnotation set\tSource\tParent\tValues\tUnit\tRaw values");
			write("-- \t-- \t---------------\t--------------\t------\t------\t------\t----\t----------");
		}
	}
	static void write_item(int i, Annotation a)
		throws BaseException
	{
		if (TestUtil.getSilent()) return;
		
		List<?> values = null;
		if (a.getSource() == Annotation.Source.PRIMARY)
		{
			values = a.getValues(null);
		}
		else
		{
			values = a.getInheritedFrom().getValues(null);
		}
		write(i+":\t"+a.getId()+"\t"+a.getAnnotationType()+"\t"+a.getAnnotationSet()+"\t"+
				a.getSource()+"\t"+a.getInheritedFrom()+"\t" +
				values + "\t" + a.getUnit() + "\t" + a.getValues());
	}
	
	static void write_item(int i, AnnotationSet as)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+as.getId()+"\t"+as.getItem());
	}

	static void write(String message)
	{
		System.out.println(message);
	}
	
	static int test_inherit_annotation(Item itemType, int itemId, int annotationId, boolean clone)
	{
		if (itemId == 0 || annotationId == 0) return 0;
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable annotatable = (Annotatable)itemType.getById(dc, itemId);
			Annotation a = Annotation.getById(dc, annotationId);
			AnnotationSet as = annotatable.getAnnotationSet();
			Annotation inherited = as.inheritAnnotation(a, clone);
			dc.commit();
			id = inherited.getId();
			dc = TestUtil.getDbControl();
			inherited = Annotation.getById(dc, id);
			write_item(0, inherited);
			write("--" + (clone ? "Clone" : "Inherit")+ " annotation OK");
		}
		catch (Throwable ex)
		{
			write("--" + (clone ? "Clone" : "Inherit")+ " annotation FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_inherit_annotationset(Item toItemType, int toItemId, Item fromItemType, int fromItemId, int expectAdded)
	{
		if (toItemId == 0 || fromItemId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotatable to = (Annotatable)toItemType.getById(dc, toItemId);
			Annotatable from = (Annotatable)fromItemType.getById(dc, fromItemId);
			AnnotationSet as = to.getAnnotationSet();
			AnnotationSet inherited = from.getAnnotationSet();
			int added = as.inheritAll(inherited, false);
			if (added != expectAdded)
			{
				throw new BaseException("Expected "+expectAdded+" results, not "+added);
			}
			dc.commit();
			write("--Inherit annotation set OK (" + added + " added)");
		}
		catch (Throwable ex)
		{
			write("--Inherit annotation set FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_modify_clone(int clonedId, int unitId, Object... value)
	{
		if (clonedId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Annotation a = Annotation.getById(dc, clonedId);
			
			List<Object> l = new ArrayList<Object>();
			l.addAll(Arrays.asList(value));
			Unit unit = unitId != 0 ? Unit.getById(dc, unitId) : null;
			a.setValues(l, unit);
			dc.commit();
			dc = TestUtil.getDbControl();
			a = Annotation.getById(dc, clonedId);
			write_item(0, a);
			write("--Modify cloned annotation OK");
		}
		catch (Throwable ex)
		{
			write("--Modify cloned annotation FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}
