/*
	$Id: TestJarClassLoader.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.BaseException;
import net.sf.basedb.util.JarClassLoader;
import java.net.URL;
import java.io.InputStreamReader;
import java.io.BufferedReader;

public class TestJarClassLoader
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing Jar class loader");
		
		test_load("JarPlugin.jar", "JarPlugin", false, true, false);
		test_resource("JarPlugin.jar", "META-INF/MANIFEST.MF", false, true);
		
		// Test parent delegation
		test_load("JarPlugin.jar", "affymetrix.calvin.exception.CalvinException", false, true, false);
		test_resource("JarPlugin.jar", "org/hibernate/hibernate-configuration-3.0.dtd", false, false);
		test_load("JarPlugin.jar", "affymetrix.calvin.exception.CalvinException", true, false, true);
		test_resource("JarPlugin.jar", "org/hibernate/hibernate-configuration-3.0.dtd", true, false);
		
		write("++Testing Jar class loader "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void test_load(String jarPath, String className, 
			boolean delegateFirst, boolean expectLoadedByJarClassLoader, boolean useNewInstance)
	{
		try
		{
			JarClassLoader cl = useNewInstance ? 
				(JarClassLoader)JarClassLoader.newInstance(jarPath) : 
				(JarClassLoader)JarClassLoader.getInstance(jarPath);
			cl.setDelegateFirst(delegateFirst);
			Class<?> c = cl.loadClass(className);
			Object o = c.newInstance();
			write_item(o);
			ClassLoader loadedBy = o.getClass().getClassLoader();
			boolean wasLoadedByJarClassLoader = loadedBy instanceof JarClassLoader;
			if (wasLoadedByJarClassLoader != expectLoadedByJarClassLoader)
			{
				throw new BaseException("Class loaded by incorrect class loader: " + loadedBy);
			}
			write("--Load class OK (" + className + ")");
		}
		catch (Throwable ex)
		{
			write("--Load class FAILED (" + className + ")");
			ex.printStackTrace();
			ok = false;
		}
	}
	
	static void test_resource(String jarPath, String name, boolean delegateFirst, boolean loadContent)
	{
		try
		{
			JarClassLoader cl = (JarClassLoader)JarClassLoader.getInstance(jarPath);
			cl.setDelegateFirst(delegateFirst);
			URL resource = cl.getResource(name);
			StringBuilder content = new StringBuilder();
			if (loadContent)
			{
				BufferedReader in = new BufferedReader(new InputStreamReader(resource.openStream()));
				String line = in.readLine();
				while (line != null)
				{
					content.append(line).append("\n");
					line = in.readLine();
				}
				in.close();
			}
			
			if (!TestUtil.getSilent())
			{
				write("Resource URL: "+resource);
				if (loadContent) write("Resource content: "+content.toString());
			}
			write("--Load resource OK (" + name + ")");
		}
		catch (Throwable ex)
		{
			write("--Load resource FAILED (" + name + ")");
			ex.printStackTrace();
			ok = false;
		}
	}

	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void write_item(Object o)
	{
		if (!TestUtil.getSilent())
		{
			Class<?> c = o.getClass();
			write("Object '"+o+"' of class '"+c.getName()+"' in package '" + c.getPackage() + 
					"' loaded with class loader: " + c.getClassLoader());
		}
	}

}

