/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Transformations = function()
{
	var transformations = {};
	
	/**
		Initialize the page.
	*/
	transformations.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', transformations.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabValidator('settings.info', transformations.validateTransformation);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var experimentId = Data.get('page-data', 'experiment-id');
			var attributes = {'item-type': 'TRANSFORMATION', 'item-id': itemId, 'extra-url': '&experiment_id='+experimentId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnCopyJob', transformations.copyJob);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);

		}
	}
	
	
	transformations.copyJob = function()
	{
		var url = '../../../common/plugin/index.jsp?ID='+App.getSessionId();
		url += '&cmd=CopyJob&job_id='+Data.get('page-data', 'job-id');
		url += '&item_type=BIOASSAYSET&context_type=LIST';
		Dialogs.openPopup(url, 'CopyJob', 750, 500);
	}

	transformations.validateTransformation = function()
	{
		var frm = document.forms['transformation'];
		if (frm.name.value == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}

	// Submit the form
	transformations.save = function()
	{
		var frm = document.forms['transformation'];
		if (TabControl.validateActiveTab('settings'))
		{
			frm.submit();
		}
	}

	return transformations;
}();

Doc.onLoad(Transformations.initPage);

